@@ -3,10 +3,8 @@
 Stub function and module used as a setuptools entry point.
 Based on augur's __main__.py and setup.py
 """
-
-from __future__ import print_function, division, absolute_import
 import sys
-from treetime import version, make_parser
+from treetime import make_parser
 
 
 # Entry point for setuptools-installed script and bin/augur dev wrapper.
