@@ -8,22 +8,25 @@ MIN_BRANCH_LENGTH = 1e-3 # fraction of length 'one_mutation' that is used as low
 OVER_DISPERSION = 10
 
 # distribution parameters
-BRANCH_GRID_SIZE_ROUGH = 200
+BRANCH_GRID_SIZE_ROUGH = 75
 NODE_GRID_SIZE_ROUGH = 60
 N_INTEGRAL_ROUGH = 60
 
-BRANCH_GRID_SIZE = 250
+BRANCH_GRID_SIZE = 125
 NODE_GRID_SIZE = 100
 N_INTEGRAL = 100
 
-BRANCH_GRID_SIZE_FINE = 300
+BRANCH_GRID_SIZE_FINE = 200
 NODE_GRID_SIZE_FINE = 180
 N_INTEGRAL_FINE = 150
 
-BRANCH_GRID_SIZE_ULTRA = 400
+BRANCH_GRID_SIZE_ULTRA = 300
 NODE_GRID_SIZE_ULTRA = 400
 N_INTEGRAL_ULTRA = 250
 
+# distribution parameters for FFT
+FFT_FWHM_GRID_SIZE = 150
+
 MIN_INTEGRATION_PEAK = 0.001
 
 # clocktree parameters
